####################################################################
#                                                                  #
# Wilfred                                                          #
# Copyright (C) 2020, Vilhelm Prytz, <vilhelm@prytznet.se>, et al. #
#                                                                  #
# Licensed under the terms of the MIT license, see LICENSE.        #
# https://github.com/wilfred-dev/wilfred                           #
#                                                                  #
####################################################################

# use "0.0.0.dev0" to indicate incremental commit build
# specify version "0.3.0" to indicate standard release
# "commit_date" should be replaced upon building
version = "0.8.0"
commit_hash = "22fb139fc1538ad229ceeaba3a57f3f174b62d16"
commit_date = "2020-12-19"
